@@ -40,10 +40,8 @@
 #define NETWORK_REGISTRATION_INTERFACE "org.ofono.NetworkRegistration"
 #define NETWORK_OPERATOR_INTERFACE "org.ofono.NetworkOperator"
 
-#define NETWORK_REGISTRATION_FLAG_REQUESTING_OPLIST 0x1
-#define NETWORK_REGISTRATION_FLAG_PENDING 0x2
-#define NETWORK_REGISTRATION_FLAG_HOME_SHOW_PLMN 0x4
-#define NETWORK_REGISTRATION_FLAG_ROAMING_SHOW_SPN 0x8
+#define NETWORK_REGISTRATION_FLAG_HOME_SHOW_PLMN 0x1
+#define NETWORK_REGISTRATION_FLAG_ROAMING_SHOW_SPN 0x2
 
 enum network_registration_mode {
 	NETWORK_REGISTRATION_MODE_AUTO = 0,
@@ -52,9 +50,6 @@ enum network_registration_mode {
 	NETWORK_REGISTRATION_MODE_MANUAL_AUTO = 4
 };
 
-/* How often we update the operator list, in seconds */
-#define OPERATOR_LIST_UPDATE_TIME 300
-
 #define SETTINGS_STORE "netreg"
 #define SETTINGS_GROUP "Settings"
 
@@ -84,7 +79,6 @@ struct ofono_netreg {
 	char *spname;
 	struct sim_spdi *spdi;
 	struct sim_eons *eons;
-	gint opscan_source;
 	struct ofono_sim *sim;
 	GKeyFile *settings;
 	char *imsi;
@@ -219,7 +213,14 @@ static void register_callback(const struct ofono_error *error, void *data)
 	netreg->pending = NULL;
 
 out:
-	netreg->flags &= ~NETWORK_REGISTRATION_FLAG_PENDING;
+	if (netreg->driver->registration_status)
+		netreg->driver->registration_status(netreg,
+					registration_status_callback, netreg);
+}
+
+static void init_register(const struct ofono_error *error, void *data)
+{
+	struct ofono_netreg *netreg = data;
 
 	if (netreg->driver->registration_status)
 		netreg->driver->registration_status(netreg,
@@ -386,6 +387,10 @@ static void set_network_operator_status(struct network_operator_data *opd,
 
 	opd->status = status;
 
+	/* Don't emit for the case where only operator name is reported */
+	if (opd->mcc[0] == '\0' && opd->mnc[0] == '\0')
+		return;
+
 	status_str = network_operator_status_to_string(status);
 	path = network_operator_build_path(netreg, opd->mcc, opd->mnc);
 
@@ -487,11 +492,6 @@ static void set_network_operator_name(struct network_operator_data *opd,
 	if (opd->eons_info && opd->eons_info->longname)
 		return;
 
-	path = network_operator_build_path(netreg, opd->mcc, opd->mnc);
-
-	ofono_dbus_signal_property_changed(conn, path, NETWORK_OPERATOR_INTERFACE,
-					"Name", DBUS_TYPE_STRING, &name);
-
 	if (opd == netreg->current_operator) {
 		const char *path = __ofono_atom_get_path(netreg->atom);
 
@@ -502,6 +502,15 @@ static void set_network_operator_name(struct network_operator_data *opd,
 						"Operator", DBUS_TYPE_STRING,
 						&operator);
 	}
+
+	/* Don't emit when only operator name is reported */
+	if (opd->mcc[0] == '\0' && opd->mnc[0] == '\0')
+		return;
+
+	path = network_operator_build_path(netreg, opd->mcc, opd->mnc);
+
+	ofono_dbus_signal_property_changed(conn, path, NETWORK_OPERATOR_INTERFACE,
+					"Name", DBUS_TYPE_STRING, &name);
 }
 
 static void set_network_operator_eons_info(struct network_operator_data *opd,
@@ -636,13 +645,12 @@ static DBusMessage *network_operator_register(DBusConnection *conn,
 	struct network_operator_data *opd = data;
 	struct ofono_netreg *netreg = opd->netreg;
 
-	if (netreg->flags & NETWORK_REGISTRATION_FLAG_PENDING)
+	if (netreg->pending)
 		return __ofono_error_busy(msg);
 
 	if (netreg->driver->register_manual == NULL)
 		return __ofono_error_not_implemented(msg);
 
-	netreg->flags |= NETWORK_REGISTRATION_FLAG_PENDING;
 	netreg->pending = dbus_message_ref(msg);
 
 	netreg->driver->register_manual(netreg, opd->mcc, opd->mnc,
@@ -782,13 +790,12 @@ static DBusMessage *network_register(DBusConnection *conn,
 {
 	struct ofono_netreg *netreg = data;
 
-	if (netreg->flags & NETWORK_REGISTRATION_FLAG_PENDING)
+	if (netreg->pending)
 		return __ofono_error_busy(msg);
 
 	if (netreg->driver->register_auto == NULL)
 		return __ofono_error_not_implemented(msg);
 
-	netreg->flags |= NETWORK_REGISTRATION_FLAG_PENDING;
 	netreg->pending = dbus_message_ref(msg);
 
 	netreg->driver->register_auto(netreg, register_callback, netreg);
@@ -803,13 +810,12 @@ static DBusMessage *network_deregister(DBusConnection *conn,
 {
 	struct ofono_netreg *netreg = data;
 
-	if (netreg->flags & NETWORK_REGISTRATION_FLAG_PENDING)
+	if (netreg->pending)
 		return __ofono_error_busy(msg);
 
 	if (netreg->driver->deregister == NULL)
 		return __ofono_error_not_implemented(msg);
 
-	netreg->flags |= NETWORK_REGISTRATION_FLAG_PENDING;
 	netreg->pending = dbus_message_ref(msg);
 
 	netreg->driver->deregister(netreg, register_callback, netreg);
@@ -819,12 +825,32 @@ static DBusMessage *network_deregister(DBusConnection *conn,
 	return NULL;
 }
 
+static DBusMessage *network_propose_scan(DBusConnection *conn,
+						DBusMessage *msg, void *data)
+{
+	struct ofono_netreg *netreg = data;
+
+	if (netreg->pending)
+		return __ofono_error_busy(msg);
+
+	if (netreg->driver->list_operators == NULL)
+		return __ofono_error_not_implemented(msg);
+
+	netreg->pending = dbus_message_ref(msg);
+
+	netreg->driver->list_operators(netreg, operator_list_callback, netreg);
+
+	return NULL;
+}
+
 static GDBusMethodTable network_registration_methods[] = {
 	{ "GetProperties",	"",	"a{sv}",	network_get_properties	},
 	{ "Register",		"",	"",		network_register,
 							G_DBUS_METHOD_FLAG_ASYNC },
 	{ "Deregister",		"",	"",		network_deregister,
 							G_DBUS_METHOD_FLAG_ASYNC },
+	{ "ProposeScan",	"",	"ao",		network_propose_scan,
+							G_DBUS_METHOD_FLAG_ASYNC },
 	{ }
 };
 
@@ -833,40 +859,6 @@ static GDBusSignalTable network_registration_signals[] = {
 	{ }
 };
 
-static void update_network_operator_list(struct ofono_netreg *netreg)
-{
-	if (netreg->flags & NETWORK_REGISTRATION_FLAG_REQUESTING_OPLIST)
-		return;
-
-	if (!netreg->driver->list_operators)
-		return;
-
-	netreg->flags |= NETWORK_REGISTRATION_FLAG_REQUESTING_OPLIST;
-	netreg->driver->list_operators(netreg, operator_list_callback, netreg);
-}
-
-static gboolean update_network_operator_list_cb(void *user_data)
-{
-	struct ofono_netreg *netreg = user_data;
-
-	update_network_operator_list(netreg);
-
-	return TRUE;
-}
-
-static gboolean update_network_operator_list_init(void *user_data)
-{
-	struct ofono_netreg *netreg = user_data;
-
-	update_network_operator_list(netreg);
-
-	netreg->opscan_source = g_timeout_add_seconds(OPERATOR_LIST_UPDATE_TIME,
-					update_network_operator_list_cb, netreg);
-
-
-	return FALSE;
-}
-
 static void set_registration_status(struct ofono_netreg *netreg, int status)
 {
 	const char *str_status = registration_status_to_string(status);
@@ -1086,23 +1078,14 @@ static GSList *compress_operator_list(const struct ofono_network_operator *list,
 	return oplist;
 }
 
-static void operator_list_callback(const struct ofono_error *error, int total,
-					const struct ofono_network_operator *list,
-					void *data)
+static gboolean update_operator_list(struct ofono_netreg *netreg, int total,
+					const struct ofono_network_operator *list)
 {
-	struct ofono_netreg *netreg = data;
 	GSList *n = NULL;
 	GSList *o;
 	GSList *compressed;
 	GSList *c;
-	gboolean need_to_emit = FALSE;
-
-	netreg->flags &= ~NETWORK_REGISTRATION_FLAG_REQUESTING_OPLIST;
-
-	if (error->type != OFONO_ERROR_TYPE_NO_ERROR) {
-		ofono_debug("Error occurred during operator list");
-		return;
-	}
+	gboolean changed = FALSE;
 
 	compressed = compress_operator_list(list, total);
 
@@ -1133,7 +1116,7 @@ static void operator_list_callback(const struct ofono_error *error, int total,
 			}
 
 			n = g_slist_prepend(n, opd);
-			need_to_emit = TRUE;
+			changed = TRUE;
 		}
 	}
 
@@ -1144,7 +1127,7 @@ static void operator_list_callback(const struct ofono_error *error, int total,
 		n = g_slist_reverse(n);
 
 	if (netreg->operator_list)
-		need_to_emit = TRUE;
+		changed = TRUE;
 
 	for (o = netreg->operator_list; o; o = o->next)
 		network_operator_dbus_unregister(netreg, o->data);
@@ -1153,6 +1136,46 @@ static void operator_list_callback(const struct ofono_error *error, int total,
 
 	netreg->operator_list = n;
 
+	return changed;
+}
+
+static void operator_list_callback(const struct ofono_error *error, int total,
+					const struct ofono_network_operator *list,
+					void *data)
+{
+	struct ofono_netreg *netreg = data;
+	DBusMessage *reply;
+	char **network_operators;
+	DBusMessageIter iter;
+	DBusMessageIter array_iter;
+	int i;
+	gboolean need_to_emit;
+
+	if (error->type != OFONO_ERROR_TYPE_NO_ERROR) {
+		ofono_debug("Error occurred during operator list");
+		__ofono_dbus_pending_reply(&netreg->pending,
+					__ofono_error_failed(netreg->pending));
+		return;
+	}
+
+	need_to_emit = update_operator_list(netreg, total, list);
+
+	reply = dbus_message_new_method_return(netreg->pending);
+	network_operator_populate_registered(netreg, &network_operators);
+
+	dbus_message_iter_init_append(reply, &iter);
+	dbus_message_iter_open_container(&iter, DBUS_TYPE_ARRAY,
+		DBUS_TYPE_OBJECT_PATH_AS_STRING, &array_iter);
+
+	for (i = 0; network_operators[i]; i++)
+		dbus_message_iter_append_basic(&array_iter,
+			DBUS_TYPE_OBJECT_PATH, &network_operators[i]);
+
+	dbus_message_iter_close_container(&iter, &array_iter);
+	__ofono_dbus_pending_reply(&netreg->pending, reply);
+
+	g_strfreev(network_operators);
+
 	if (need_to_emit)
 		network_operator_emit_available_operators(netreg);
 }
@@ -1213,7 +1236,8 @@ static void current_operator_callback(const struct ofono_error *error,
 
 		opd = network_operator_create(current);
 
-		if (!network_operator_dbus_register(netreg, opd)) {
+		if (opd->mcc[0] != '\0' && opd->mnc[0] != '\0' &&
+				!network_operator_dbus_register(netreg, opd)) {
 			g_free(opd);
 			return;
 		}
@@ -1222,7 +1246,8 @@ static void current_operator_callback(const struct ofono_error *error,
 		netreg->operator_list = g_slist_append(netreg->operator_list,
 							opd);
 
-		network_operator_emit_available_operators(netreg);
+		if (opd->mcc[0] != '\0' && opd->mnc[0] != '\0')
+			network_operator_emit_available_operators(netreg);
 	} else {
 		/* We don't free this here because operator is registered */
 		/* Taken care of elsewhere */
@@ -1281,7 +1306,7 @@ static void init_registration_status(const struct ofono_error *error,
 		(status == NETWORK_REGISTRATION_STATUS_NOT_REGISTERED ||
 			status == NETWORK_REGISTRATION_STATUS_DENIED ||
 			status == NETWORK_REGISTRATION_STATUS_UNKNOWN))
-		netreg->driver->register_auto(netreg, register_callback, netreg);
+		netreg->driver->register_auto(netreg, init_register, netreg);
 }
 
 void ofono_netreg_strength_notify(struct ofono_netreg *netreg, int strength)
@@ -1578,11 +1603,6 @@ static void netreg_unregister(struct ofono_atom *atom)
 	__ofono_watchlist_free(netreg->status_watches);
 	netreg->status_watches = NULL;
 
-	if (netreg->opscan_source) {
-		g_source_remove(netreg->opscan_source);
-		netreg->opscan_source = 0;
-	}
-
 	for (l = netreg->operator_list; l; l = l->next)
 		network_operator_dbus_unregister(netreg, l->data);
 
@@ -1722,10 +1742,6 @@ void ofono_netreg_register(struct ofono_netreg *netreg)
 
 	ofono_modem_add_interface(modem, NETWORK_REGISTRATION_INTERFACE);
 
-	if (netreg->driver->list_operators)
-		netreg->opscan_source = g_timeout_add_seconds(5,
-				update_network_operator_list_init, netreg);
-
 	if (netreg->driver->registration_status)
 		netreg->driver->registration_status(netreg,
 					init_registration_status, netreg);
